/*-----------------------------------------------------------------------------
  Project:  QRE
  File:     WBMHandler.cpp

  Copyright (C)2010 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

  $Revision: 1.6 $
 *----------------------------------------------------------------------------*/

/// @file
//
//
/// @version 1.0

#include "WBMHandler.h"

//
///
/// Load bitmap header information from specified stream.
//
//
//
int BitMapCoreHeader::read_header(ISTREAM &is)
{
    // Buffer size for size information changes depending on header type, so call method to load size information for each class.
    // 
    int ret = read_sizeinfo(is);

    if (!ret) {
        // Load remaining information if size information was successfully loaded.
        ReadFile(is, &plane);
        ReadFile(is, &color_bits);
    }
    return ret;
}

//
///
/// Save bitmap header information to specified stream.
//
//
//
int BitMapCoreHeader::write_header(OSTREAM &os)
{
    WriteFile(os, &header_size);

    // Buffer size for size information changes depending on header type, so call method to save size information for each class.
    // 
    int ret = write_sizeinfo(os);

    if (!ret) {
        // Save remaining information if size information was successfully saved.
        WriteFile(os, &plane);
        WriteFile(os, &color_bits);
    }
    return ret;
}

//
///
/// Buffer size for size information is 2 bytes for OS2 version 1 and 4 bytes otherwise, so change loading method depending on header type.
/// 
//
//
//
int BitMapCoreHeader::read_sizeinfo(ISTREAM &is)
{
    uint16_t uwork;
    ReadFile(is, &uwork);
    width = static_cast<uint32_t>(uwork);
    int16_t swork;
    ReadFile(is, &swork);
    height = static_cast<int32_t>(swork);

    return 0;
}

//
///
/// Change function to save information according to header type, just as when loading.
//
//
//
int BitMapCoreHeader::write_sizeinfo(OSTREAM &os)
{
    uint16_t uwork = static_cast<uint16_t>(width);
    WriteFile(os, &uwork);
    int16_t swork = static_cast<int16_t>(height);
    WriteFile(os, &swork);

    return 0;
}

//
///
/// Load bitmap header information from specified stream.
//
//
//
int BitMapInfoHeader::read_header(ISTREAM &is)
{
    int ret = BitMapCoreHeader::read_header(is);
    if (ret)
        return ret;

    // Save class members.
    ReadFile(is, &compress);
    ReadFile(is, &image_size);
    ReadFile(is, &horizontal_PPM);
    ReadFile(is, &vertical_PPM);
    ReadFile(is, &num_of_color);
    ReadFile(is, &important_color);

    return 0;
}

//
///
/// Save bitmap header information to specified stream.
//
//
//
int BitMapInfoHeader::write_header(OSTREAM &os)
{
    int ret = BitMapCoreHeader::write_header(os);
    if (ret)
        return ret;

    // Save class members.
    WriteFile(os, &compress);
    WriteFile(os, &image_size);
    WriteFile(os, &horizontal_PPM);
    WriteFile(os, &vertical_PPM);
    WriteFile(os, &num_of_color);
    WriteFile(os, &important_color);

    return 0;
}

//
///
/// Change loading method depending on header type, just as for BITMAPCOREHEADER class methods.
/// 
//
//
//
int BitMapInfoHeader::read_sizeinfo(ISTREAM &is)
{
    ReadFile(is, &width);
    ReadFile(is, &height);

    return 0;
}

//
///
/// Change function to save information according to header type, just as when loading.
//
//
//
int BitMapInfoHeader::write_sizeinfo(OSTREAM &os)
{
    WriteFile(os, &width);
    WriteFile(os, &height);

    return 0;
}

//
///
/// Load bitmap basic information from specified stream.
//
//
//
int BitMapFileHeader::read_header(ISTREAM &is)
{
    // Load class members
    size_t read_size = sizeof(char) * sizeof(magic);
    ReadFile(is, &magic, read_size);
    if (magic[0] != 'B' || magic[1] != 'M')
        return -1;

    ReadFile(is, &size);
    ReadFile(is, &reserve_1);
    ReadFile(is, &reserve_2);
    ReadFile(is, &offset);

    // Get bitmap header size
    uint32_t header_size;
    ReadFile(is, &header_size);

    if (core_header) delete core_header;
    // Choose class to generate depending on size
    switch (header_size) {
    case BHS_OS2_V1_SIZE:
        core_header = new BitMapCoreHeader();
        break;
    case BHS_WIN_V3_SIZE:
        core_header
            = reinterpret_cast<BitMapCoreHeader *>(new BitMapInfoHeader());
        break;
    default:
        core_header = NULL;
        break;
    }
    if (!core_header) {
        return -1;
    }
    int ret = core_header->read_header(is);

    // Finishing normally, if we have the color map size...
    if (ret == 0) {
        color_map_size = offset - BITMAPFILEHEADER_SIZE - header_size;
        // 
        if (color_map_size > 0) {
            if (color_map)
                delete [] color_map;
            color_map = new uint8_t[color_map_size];
            if (!color_map)
                return -1;

            ReadFile(is, color_map, color_map_size);
        }
    }
    return ret;
}

//
///
/// Save bitmap basic information to specified stream.
//
//
//
int BitMapFileHeader::write_header(OSTREAM &os)
{
    if (!core_header || (color_map_size && !color_map))
        return -1;

    // Save class members.
    WriteFile(os, magic, sizeof(char) * sizeof(magic));
    WriteFile(os, &size);
    WriteFile(os, &reserve_1);
    WriteFile(os, &reserve_2);
    WriteFile(os, &offset);

    // Save bitmap header.
    int ret = core_header->write_header(os);
    // If saving succeeded and we need to save color map...
    if (ret == 0 && color_map_size)
        WriteFile(os, color_map, color_map_size);

    return ret;
}

/* EOF */
